<?php

namespace Includes\Controllers\Api;

use Includes\baseClasses\KCBase;
use TenQuality\WP\Database\QueryBuilder;

use App\models\KCAppointment;

use App\models\KCAppointmentServiceMapping;
use Includes\model\KCTaxModel;
use WP_Error;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

class KCAppointmentController extends KCBase
{

	public $module = 'appointment';

	public $nameSpace;

	function __construct()
	{

		$this->nameSpace = KIVICARE_API_NAMESPACE;

		add_action('rest_api_init', function () {

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/get-appointment', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'get_appointment'],
				'permission_callback' => '__return_true',
			));


			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/get-appointment-slot', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'get_appointment_slots'],
				'permission_callback' => '__return_true',
			));

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/update-status', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'updateStatus'],
				'permission_callback' => '__return_true',
			));

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/save', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'saveAppointment'],
				'permission_callback' => '__return_true',
			));

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/delete', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'deleteAppointment'],
				'permission_callback' => '__return_true',
			));

			register_rest_route($this->nameSpace . '/api/v2/' . $this->module, '/save', array(
				'methods'             => WP_REST_Server::ALLMETHODS,
				'callback'            => [$this, 'saveAppointmentWithFile'],
				'permission_callback' => '__return_true',
			));

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/payment-status', array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => [$this, 'changeAppointmentStatusOnPayment'],
				'permission_callback' => [$this, 'isValidToken'],
				'args'                => array(
					'status'      => array(
						'description' => __('Payment Status', 'kivicare-api'),
						'type'     => 'string',
						'required' => true,
						'enum'     => array('captured', 'failed'),
					),
					'appointment_id'      => array(
						'description' => __('Appointment ID', 'kivicare-api'),
						'required'    => true,
						'type'        => 'int',
						'sanitize_callback' => 'absint',
					),
				),
			));
		});
	}
	/**
	 * The function changes the status of an appointment on Razorpay and performs various actions based on
	 * the status, such as sending appointment notifications and updating appointment and payment
	 * statuses.
	 * 
	 * @param WP_REST_Request request The  parameter is an instance of the WP_REST_Request class,
	 * which is used to handle REST API requests in WordPress. It contains information about the request,
	 * such as the request method, headers, and parameters. In this function, it is used to retrieve the
	 * values of the parameters passed in the
	 * 
	 * @return null response in the form of a JSON object. If the status parameter is 'approved', it will
	 * return an empty array with a status code of 200. If the status parameter is not 'approved', it will
	 * return an empty string with a status code of 204.
	 */
	public function changeAppointmentStatusOnPayment(WP_REST_Request $request)
	{
		global $wpdb;
		if ($request->get_param('status') === 'captured') {
			//check if appointment notification not already sent
			if (empty($wpdb->get_var("SELECT notification_status FROM {$wpdb->prefix}kc_payments_appointment_mappings WHERE appointment_id =" . $request->get_param('appointment_id')))) {
				// send appointment notification
				kivicareWoocommercePaymentComplete($request->get_param('appointment_id'), 'razorpay');
				//update appointment status to booked (1)
				$wpdb->update($wpdb->prefix . "kc_appointments", ['status' => 1], ['id' => $request->get_param('appointment_id')]);
				//update appointment payment status to approved
				$wpdb->update($wpdb->prefix . "kc_payments_appointment_mappings", [
					'payment_status' => 'approved',
				], ['appointment_id' => $request->get_param('appointment_id')]);
			}
			return comman_custom_response(["message"=> __("Payment Successfull",'kivicare-api')], 200);
		} else {
			//delete all appointment related data if payment failed
			$wpdb->delete($wpdb->prefix . "kc_payments_appointment_mappings", ['appointment_id' => $request->get_param('appointment_id')]);
			(new KCAppointment())->delete(['id' => $request->get_param('appointment_id')]);
			(new KCAppointmentServiceMapping())->delete(['appointment_id' => $request->get_param('appointment_id')]);
			return comman_custom_response(["message"=> __("Something Went Wrong",'kivicare-api')], 204);
		}
	}

	public function get_appointment($request)
	{

		global $wpdb;

		$data = $this->kcaCheckAuthenticateUser($request);

		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}

		$role = $data['role'];
		$userid = $data['user_id'];

		$parameters = $request->get_params();

		$parameters['isKiviCareProOnName'] = $this->isKiviCareProOnName();

		$appointment_list = kcaAppointmentList($parameters, $data);

		return comman_custom_response($appointment_list);
	}

	public function get_appointment_slots($request)
	{

		$data = $this->kcaCheckAuthenticateUser($request);

		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}

		$paramaters = $request->get_params();

		$rules = [
			'date'      => 'required',
			'clinic_id' => 'required',
			'doctor_id' => 'required',
		];

		$message = [
			'clinic_id' => esc_html__('Clinic is required', 'kca-lang'),
			'doctor_id' => esc_html__('Doctor is required', 'kca-lang'),
		];

		$validation = kcaValidateRequest($rules, $paramaters, $message);

		if (count($validation)) {
			return comman_message_response($validation[0], 400);
		}
	}

	public function updateStatus($request)
	{
		global $wpdb;

		$data = $this->kcaCheckAuthenticateUser($request);

		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}
		$userid = $data['user_id'];
		$parameters = $request->get_params();
		$appointment_table = $wpdb->prefix . 'kc_' . 'appointments';
		$rules  = [
			'appointment_id'     => 'required',
			'appointment_status' => 'required',
		];

		$validation = kcaValidateRequest($rules, $parameters);

		if (count($validation)) {
			return comman_message_response($validation[0], 400);
		}

		do_action(
			"kc_before_send_email",
			[
				'patient_name' => " ",
				"doctor_name" => " ",
				"clinic" => " ",
				"user_email" => ""
			],
			[
				"4" => "patient_clinic_check_in",
				"3" => "patient_clinic_check_out"
			][$parameters['appointment_status']]
		);

		$wpdb->update($appointment_table, ['status' => $parameters['appointment_status']], array('id' => $parameters['appointment_id']));

		if ($parameters['appointment_status'] === '2' || $parameters['appointment_status'] === '4') {
			// Create Encounter
			$result = $this->createEncounter($parameters['appointment_id'], $userid);
		}

		if ($parameters['appointment_status'] === '3' || $parameters['appointment_status'] === '0') {
			// Close Encounter0
			$result = $this->closeEncounter($parameters['appointment_id']);
		}
		$status_code = 200;
		$message = __("Appointment status has been updated successfully", 'kivicare-api');
		if (!$result) {
			$message = __("Appointment status update failed", 'kivicare-api');
			$status_code = 400;
		}

		return comman_message_response($message, $status_code);
	}

	public function createEncounter($appointment_id, $userid)
	{
		global $wpdb;

		$appointment_id = (isset($appointment_id) && $appointment_id != '') ? $appointment_id : null;

		$appointment_table  = $wpdb->prefix . 'kc_' . 'appointments';
		$encounter_table    = $wpdb->prefix . 'kc_' . 'patient_encounters';
		$service_table		= $wpdb->prefix . 'kc_' . 'services';
		$service_mapping_table = $wpdb->prefix . 'kc_' . 'service_doctor_mapping';
		$appointment_service_table	= $wpdb->prefix . 'kc_' . 'appointment_service_mapping';
		$query = " SELECT * FROM {$appointment_table} WHERE id = {$appointment_id}";
		$appointment = $wpdb->get_row($query, OBJECT);

		$encounter = $wpdb->get_results(" SELECT * FROM {$encounter_table} WHERE appointment_id = {$appointment_id}", OBJECT);

		if ($encounter === []) {
			$temp = [
				"appointment_id" => $appointment_id,
				"encounter_date" => date('Y-m-d'),
				"clinic_id" => $appointment->clinic_id,
				"doctor_id" => $appointment->doctor_id,
				"patient_id" => $appointment->patient_id,
				"description" => $appointment->description,
				"added_by" => $userid,
				"status"  => 1,
				"created_at" => current_time('Y-m-d H:i:s')
			];

			$encounter_result = $wpdb->insert($encounter_table, $temp);
			/*
			$config = kcaGetModules();

			$modules = collect($config->module_config)->where('name','billing')->where('status', 1)->count();

		if($modules > 0){*/
			$appointment_services = $wpdb->get_results("SELECT service_id FROM {$appointment_service_table} WHERE appointment_id = {$appointment_id} ", OBJECT);

			$service_ids = collect($appointment_services)->pluck('service_id')->implode(',');
			$service_data = $wpdb->get_results(" SELECT * FROM {$service_mapping_table} WHERE doctor_id = {$appointment->doctor_id} AND service_id IN ($service_ids) AND clinic_id LIKE {$appointment->clinic_id}   ", OBJECT);


			$total_amount = 0;
			foreach ($service_data as $service) {
				$total_amount = $total_amount + $service->charges;
				$bill_item_data[] = [
					'id'		=> "",
					'item_id'	=> $service->service_id,
					'price'		=> $service->charges,
					'qty'		=> 1,
					'bill_id'	=> "",
				];
			}

			// print_r($bill_item_data);die;
			$bill_data = [
				'encounter_id'  => $wpdb->insert_id,
				'appointment_id' => $appointment_id,
				'total_amount'  => $total_amount,
				'discount'      => 0,
				'actual_amount' => $total_amount,
				'status'        => (isset($parameters['status']) && $parameters['status'] != '') ? $parameters['status'] : 0,
				'payment_status' => (isset($parameters['payment_status']) && $parameters['payment_status'] != '') ? $parameters['payment_status'] : null,
				'created_at'	=> current_time('Y-m-d H:i:s'),
			];
			$bill_data['billItems'] = $bill_item_data;
			$bill_data['isKiviCareProOnName'] = $this->isKiviCareProOnName();
			kcaGenerateBill($bill_data);
			// }
			return true;
		} else {
			return false;
		}
	}

	public function closeEncounter($appointment_id)
	{
		global $wpdb;

		$appointment_id = (isset($appointment_id) && $appointment_id != '') ? $appointment_id : null;

		$encounter_table = $wpdb->prefix . 'kc_' . 'patient_encounters';

		$encounter = $wpdb->get_row(" SELECT * FROM {$encounter_table} WHERE appointment_id = {$appointment_id}", OBJECT);

		if ($encounter !== []) {
			$wpdb->update($encounter_table, ['status' => '0'], ['id' => $encounter->id]);
			return true;
		}
		return false;
	}

	public function saveAppointment($request)
	{
		global $wpdb;

		$data = $this->kcaCheckAuthenticateUser($request);

		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}

		$parameters = $request->get_params();
		$appointment_table 		= $wpdb->prefix . 'kc_' . 'appointments';
		$service_table 		= $wpdb->prefix . 'kc_' . 'services';
		$appointment_service_table	= $wpdb->prefix . 'kc_' . 'appointment_service_mapping';
		$encounter_table   		= $wpdb->prefix . 'kc_' . 'patient_encounters';
		$clinic_session_table 	= $wpdb->prefix . 'kc_' . 'clinic_sessions';
		$role 	= $data['role'];
		$userid = $data['user_id'];

		/* 
		if ( $role == 'doctor' ) {
			$parameters['doctor_id'] = $userid;
		}

		if ( $role == 'patient' ) {
			$parameters['patient_id'] = $userid;
		}
		*/
		$rules = [
			'appointment_start_date' => 'required',
			'appointment_start_time' => 'required',
			'visit_type'             => 'required',
			'clinic_id'              => 'required',
			'doctor_id'              => 'required',
			'patient_id'             => 'required',
			'status'                 => 'required',
		];

		$message = [
			'status'     => 'Status is required',
			'patient_id' => 'Patient is required',
			'clinic_id'  => 'Clinic is required',
			'doctor_id'  => 'Doctor is required',
			'visit_type' => 'Service type is required'
		];

		$validation = kcaValidateRequest($rules, $parameters, $message);

		if (count($validation)) {
			return comman_message_response($validation[0], 400);
		}
		$day = strtolower(date('D', strtotime($parameters['appointment_start_time'])));

		$clinic_session = $wpdb->get_row(" SELECT * FROM {$clinic_session_table} WHERE clinic_id = {$parameters['clinic_id']} AND doctor_id = {$parameters['doctor_id']} AND day = '{$day}' ", OBJECT);

		$time_slot = isset($clinic_session->time_slot) ? $clinic_session->time_slot : 15;

		$end_time             = strtotime("+" . $time_slot . " minutes", strtotime($parameters['appointment_start_time']));
		$appointment_end_time = date('H:i:s', $end_time);
		$appointment_date     = date('Y-m-d', strtotime($parameters['appointment_start_date']));

		$temp = [
			'appointment_start_date' => $appointment_date,
			'appointment_start_time' => date('H:i:s', strtotime($parameters['appointment_start_time'])),
			'appointment_end_date'   => $appointment_date,
			'appointment_end_time'   => $appointment_end_time,
			'clinic_id'              => $parameters['clinic_id'],
			'doctor_id'              => $parameters['doctor_id'],
			'patient_id'             => $parameters['patient_id'],
			'description'            => $parameters['description'],
			'status'                 => $parameters['status']
		];

		if (isset($parameters['id']) && $parameters['id'] !== "") {
			$appointment_id = $parameters['id'];
			$wpdb->update($appointment_table, $temp, array('id' => $appointment_id));

			$encounter_data = [
				'encounter_date'	=> $appointment_date,
				'patient_id'		=> $parameters['patient_id'],
				'doctor_id'			=> $parameters['doctor_id'],
				'clinic_id'			=> $parameters['clinic_id'],
				'description'		=> $parameters['description']
			];
			$wpdb->update($encounter_table, $encounter_data, ['appointment_id' => $appointment_id]);

			$wpdb->delete($appointment_service_table, ['appointment_id' => $appointment_id]);
			$message = __('Appointment has been updated successfully', 'kivicare-api');
		} else {

			$temp['created_at'] = current_time('Y-m-d H:i:s');
			$wpdb->insert($appointment_table, $temp);
			$appointment_id = $wpdb->insert_id;
			$message = __('Appointment has been saved successfully', 'kivicare-api');

			// if appointment is not successfully created. (WP Error handle) 
			if (is_wp_error($appointment_id) || $appointment_id == false || $appointment_id == 0) {
				$message = __('Appointment is not successfully booked.', 'kivicare-api');
				return comman_message_response($message, 400);
			}
		}

		$service_data_parameter = [];

		foreach ($parameters['visit_type'] as $service) {

			$service_data = $wpdb->get_row("SELECT * FROM {$service_table} WHERE id = {$service} ", OBJECT);

			if ($service_data != null) {
				array_push($service_data_parameter, [
					'id' => $service_data->id,
					'name' => $service_data->name
				]);
			}
		}
		// email send
		if ($parameters['status'] == 1) {

			if (!$this->isKiviCareProOnName() && !$this->teleMedAddOnName()) {
				foreach ($parameters['visit_type'] as $key => $value) {
					$service = strtolower($value['name']);
					if ($service != 'telemed') {
						$patient_email_status = kcAppointmentPatientEmail($appointment_id, kcServiceListFromRequestData($service_data_parameter));
					}
				}
				$doctor_email_status = kcAppointmentDoctorEmail($appointment_id, kcServiceListFromRequestData($service_data_parameter));
			}
		}

		foreach ($parameters['visit_type'] as $service) {

			$service_data = $wpdb->get_row("SELECT * FROM {$service_table} WHERE id = {$service['id']} ", OBJECT);

			if ($service_data != null) {
				if (strtolower($service_data->name) === 'telemed') {

					if ($this->teleMedAddOnName()) {
						$parameters['isKiviCareProOnName'] = $this->isKiviCareProOnName();
						$parameters['appointment_id'] = $appointment_id;
						$parameters['time_slot'] = $time_slot;

						$res_data = kctCreateAppointmentMeeting($parameters);
					}
				}
			}

			$service_data = [
				'appointment_id' => $appointment_id,
				'service_id'	=> $service['id'],
				'created_at' 	=> isset($parameters['id']) ? current_time('Y-m-d H:i:s') : null,
				'status'		=> 1
			];
			$result = $wpdb->get_results("SELECT * FROM $appointment_service_table WHERE appointment_id = $appointment_id AND service_id={$service['id']} ");
			$wpdb->insert($appointment_service_table, $service_data);
		}

		if ($parameters['status'] === '2' || $parameters['status'] === '4') {
			$this->createEncounter($appointment_id, $userid);
		}

		if ($this->isKiviCareProOnName() || $this->teleMedAddOnName()) {
			function_exists("kcAppointmentSendMailBasedOnWoo") && kcAppointmentSendMailBasedOnWoo($appointment_id, $parameters);
		}

		$message = __('Appointment booked successfully, Please check your email.', 'kivicare-api');

		// woocommerce payment or  telemed cart page addon
		$parameters['doctor_id'] = [
			'id' => $parameters['doctor_id']
		];
		if ($parameters['id'] == null && $parameters['id'] == "") {
			$woocommerce_response = kcWoocommerceRedirect($appointment_id, $parameters);
		}

		if ($woocommerce_response['status']) {
			$response['status'] = $woocommerce_response['status'];
			$response['message'] = $message;
			$response['woocommerce_redirect'] = $woocommerce_response['woocommerce_cart_data']['woocommerce_redirect'];

			return comman_custom_response($response);
		}

		return comman_message_response($message);
	}

	public function saveAppointmentWithFile($request)
	{
		global $wpdb;

		$data = $this->kcaCheckAuthenticateUser($request);


		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}

		$parameters = $request->get_params();

		$appointment_table 		= $wpdb->prefix . 'kc_' . 'appointments';
		$service_table 		= $wpdb->prefix . 'kc_' . 'services';
		$appointment_service_table	= $wpdb->prefix . 'kc_' . 'appointment_service_mapping';
		$encounter_table   		= $wpdb->prefix . 'kc_' . 'patient_encounters';
		$clinic_session_table 	= $wpdb->prefix . 'kc_' . 'clinic_sessions';
		$role 	= $data['role'];
		$userid = $data['user_id'];

		/* 
		if ( $role == 'doctor' ) {
			$parameters['doctor_id'] = $userid;
		}

		if ( $role == 'patient' ) {
			$parameters['patient_id'] = $userid;
		}
		*/
		$rules = [
			'appointment_start_date' => 'required',
			'appointment_start_time' => 'required',
			'visit_type'             => 'required',
			'clinic_id'              => 'required',
			'doctor_id'              => 'required',
			'patient_id'             => 'required',
			'status'                 => 'required',
		];

		$message = [
			'status'     => 'Status is required',
			'patient_id' => 'Patient is required',
			'clinic_id'  => 'Clinic is required',
			'doctor_id'  => 'Doctor is required',
			'visit_type' => 'Service type is required'
		];

		$validation = kcaValidateRequest($rules, $parameters, $message);

		if (count($validation)) {
			return comman_message_response($validation[0], 400);
		}
		$day = strtolower(date('D', strtotime($parameters['appointment_start_time'])));

		$clinic_session = $wpdb->get_row(" SELECT * FROM {$clinic_session_table} WHERE clinic_id = {$parameters['clinic_id']} AND doctor_id = {$parameters['doctor_id']} AND day = '{$day}' ", OBJECT);

		$time_slot = isset($clinic_session->time_slot) ? $clinic_session->time_slot : 15;

		$end_time             = strtotime("+" . $time_slot . " minutes", strtotime($parameters['appointment_start_time']));
		$appointment_end_time = date('H:i:s', $end_time);
		$appointment_date     = date('Y-m-d', strtotime($parameters['appointment_start_date']));
		$attachment_count = isset($parameters['attachment_count']) ? $parameters['attachment_count'] : '';
		$appointment_report_id = [];
		if ($attachment_count != null) {
			for ($i = 0; $i < $attachment_count; $i++) {
				$attachment = "appointment_report_" . $i;
				if ($_FILES[$attachment] != null) {
					array_push($appointment_report_id, media_handle_upload($attachment, 0));
				}
			}
		}

		$temp = [
			'appointment_start_date' => $appointment_date,
			'appointment_start_time' => date('H:i:s', strtotime($parameters['appointment_start_time'])),
			'appointment_end_date'   => $appointment_date,
			'appointment_end_time'   => $appointment_end_time,
			'clinic_id'              => $parameters['clinic_id'],
			'doctor_id'              => $parameters['doctor_id'],
			'patient_id'             => $parameters['patient_id'],
			'description'            => $parameters['description'],
			'status'                 => $parameters['status'],
			'appointment_report'	 => json_encode($appointment_report_id)
		];

		if (isset($parameters['id']) && $parameters['id'] !== "") {
			$appointment_id = $parameters['id'];
			$wpdb->update($appointment_table, $temp, array('id' => $appointment_id));

			$encounter_data = [
				'encounter_date'	=> $appointment_date,
				'patient_id'		=> $parameters['patient_id'],
				'doctor_id'			=> $parameters['doctor_id'],
				'clinic_id'			=> $parameters['clinic_id'],
				'description'		=> $parameters['description']
			];
			$wpdb->update($encounter_table, $encounter_data, ['appointment_id' => $appointment_id]);

			// $wpdb->delete($appointment_service_table, ['appointment_id' => $appointment_id]);
			$message = __('Appointment has been updated successfully', 'kivicare-api');
		} else {

			$temp['created_at'] = current_time('Y-m-d H:i:s');

			$checkAppointmentData =  $wpdb->get_row("SELECT * FROM {$appointment_table} WHERE appointment_start_date='{$temp['appointment_start_date']}' AND appointment_start_time='{$temp['appointment_start_time']}' AND appointment_end_date='{$temp['appointment_end_date']}' AND appointment_end_time='{$temp['appointment_end_time']}' AND clinic_id={$temp['clinic_id']} AND doctor_id={$temp['doctor_id']} AND status = 1 ");
			if (!empty($checkAppointmentData)) {
				return comman_custom_response(['message' => __('Appointment Already Booked For This Time Slot.', 'kivicare-api')], 400);
			}

			$wpdb->insert($appointment_table, $temp);
			$appointment_id = $wpdb->insert_id;
			$message = __('Appointment has been saved successfully','kivicare-api');

			// if appointment is not successfully created. (WP Error handle) 
			if (is_wp_error($appointment_id) || $appointment_id == false || $appointment_id == 0) {
				$message = __('Appointment is not successfully booked.', 'kivicare-api');
				return comman_message_response($message, 400);
			}
		}

		$service_ids_csv = implode(",", $parameters['visit_type']);

		$parameters['visit_type'] =	$service_data_parameter = QueryBuilder::create()
			->select("DISTINCT " . KC_SERVICE_TABLE . ".* ," . KC_SERVICE_DOCTOR_MAPPING_TABLE . ".telemed_service ," . KC_SERVICE_DOCTOR_MAPPING_TABLE . ".id AS `service_doctor_id`")
			->from(KC_SERVICE_TABLE, false)
			->join(KC_SERVICE_DOCTOR_MAPPING_TABLE, [
				[
					"key_a" => KC_SERVICE_DOCTOR_MAPPING_TABLE . ".service_id",
					"key_b" => KC_SERVICE_TABLE . ".id"
				],
			], "LEFT", false)
			->where(['raw' => KC_SERVICE_DOCTOR_MAPPING_TABLE . ".service_id IN ({$service_ids_csv}) AND " . KC_SERVICE_DOCTOR_MAPPING_TABLE . ".clinic_id = {$parameters['clinic_id']} AND " . KC_SERVICE_DOCTOR_MAPPING_TABLE . ".doctor_id = {$parameters['doctor_id']}"])
			->get(ARRAY_A);


		// email send
		if ($parameters['status'] != 0) {
			// Telemed appointment booking no need to send normal appointment booking email
			//send email only if lite plugin is active
			$data_visit_type = $parameters['visit_type'];
			$names_visit_type = [];
			foreach ($data_visit_type as $item) {
				if (isset($item['name'])) {
					$names_visit_type[] = $item['name'];
				}
			}
			$names_visit_type =  '"'.implode(',', $names_visit_type).'"';
			$notification = kivicareCommonSendEmailIfOnlyLitePluginActive( $appointment_id,$names_visit_type);
		}

		if ($parameters['status'] == 1) {

			if (!$this->isKiviCareProOnName() && !$this->teleMedAddOnName()) {
				foreach ($parameters['visit_type'] as $key => $value) {
					$service_data = $wpdb->get_row("SELECT * FROM {$service_table} WHERE id = {$value['id']} ", OBJECT);
					$service = strtolower($service_data->name);
					if ($service != 'telemed') {
						$patient_email_status = kcAppointmentPatientEmail($appointment_id, kcServiceListFromRequestData($service_data_parameter));
					}
				}
				$doctor_email_status = kcAppointmentDoctorEmail($appointment_id, kcServiceListFromRequestData($service_data_parameter));
			}
		}


		foreach ($parameters['visit_type'] as $service) {

			if ($service['telemed_service'] == "yes") {

				if ($this->teleMedAddOnName() || isKiviCareGooglemeetActive()) {
					$parameters['appointment_id'] = $appointment_id;
					$parameters['time_slot'] = $time_slot;

					if (kcCheckDoctorTelemedType($appointment_id) == 'googlemeet') {
						$res_data = apply_filters('kcgm_save_appointment_event', [
							'appoinment_id' => $appointment_id,
							'service' => kcServiceListFromRequestData($service_data_parameter)
						]);
					} else {
						$parameters['isKiviCareProOnName'] = $this->isKiviCareProOnName();
						$res_data = kctCreateAppointmentMeeting($parameters);
						// $res_data = apply_filters('kct_create_appointment_meeting', $parameters);
					}

					$process_status['telemed']['status'] = false;
					$process_status['telemed']['message'] = $res_data['message'];

					// Handle Invalid zoom link access token
					if (!empty($res_data) && $res_data['status'] == true) {
						$telemed_link_send = false;
						$process_status['telemed']['status'] = true;
						$process_status['telemed']['message'] = $res_data['message'];
						$process_status['telemed']['join_url'] = $res_data['data']['join_url'];

						if (get_option(KIVI_CARE_PREFIX . 'woocommerce_payment') == false || get_option(KIVI_CARE_PREFIX . 'woocommerce_payment') != 'on') {
							if (kcCheckDoctorTelemedType($appointment_id) == 'googlemeet') {
								$telemed_link_send = apply_filters('kcgm_save_appointment_event_link_send', ['appoinment_id' => $appointment_id]);
							} else {
								$telemed_link_send = apply_filters('kct_send_zoom_link', ['appointment_id' => $appointment_id]);
							}
						} else {
							//send email if woocommerce enable and status of order is completed
							if (
								isset($request_data['id']) && $request_data['id'] != ""
								&& ($active_domain === $this->kiviCareProOnName() || $this->isTeleMedActive())
							) {
								if (kcAppointmentWoocommerceOrderStatus($appointment_id) || $this->getLoginUserRole() !== $this->getPatientRole()) {
									if (kcCheckDoctorTelemedType($appointment_id) == 'googlemeet') {
										$telemed_link_send = apply_filters('kcgm_save_appointment_event_link_send', ['appoinment_id' => $appointment_id]);
									} else {
										$telemed_link_send = apply_filters('kct_send_zoom_link', ['appointment_id' => $appointment_id]);
									}
								}
							}
						}
						// Email zoom link send status 
						$process_status['telemed']['link_send_status'] = $telemed_link_send;
					}
				}
				/* if ($this->teleMedAddOnName()) {
						$parameters['isKiviCareProOnName'] = $this->isKiviCareProOnName();
						$parameters['appointment_id'] = $appointment_id;
						$parameters['time_slot'] = $time_slot;
						
						$res_data = kctCreateAppointmentMeeting($parameters);
					} */
			}

			// echo json_encode($service['telemed_service'] =="yes");


			$service_data = [
				'appointment_id' => $appointment_id,
				'service_id'	=> $service['id'],
				'created_at' 	=> !isset($parameters['id']) ? current_time('Y-m-d H:i:s') : null,
				'status'		=> 1
			];

			if (!isset($parameters['id'])) {
				$wpdb->insert($appointment_service_table, $service_data);
			}
		}

		if ($parameters['status'] === '2' || $parameters['status'] === '4') {
			$this->createEncounter($appointment_id, $userid);
		}
		$notification = '';
		if ($this->isKiviCareProOnName() || $this->teleMedAddOnName() || isKiviCareGooglemeetActive()) {
			$notification = function_exists("kcAppointmentSendMailBasedOnWoo") && kcAppointmentSendMailBasedOnWoo($appointment_id, $parameters);
		}

		$message = __('Appointment booked successfully, Please check your email.', 'kivicare-api');

		// woocommerce payment or  telemed cart page addon
		$parameters['doctor_id'] = [
			'id' => $parameters['doctor_id']
		];


		switch ($parameters['payment_mode']) {
			case 'paymentRazorpay':
				$wpdb->update($wpdb->prefix . "kc_appointments", ['status' => 2], ['id' => $appointment_id]);
				$parameters['appointment_id'] = $appointment_id;

				$parameters['service_id']= [];
				foreach ($parameters['visit_type'] as $key => $value) {
					$parameters['visit_type'][$key]['service_id'] = $parameters['visit_type'][$key]['id'];
					$parameters['visit_type'][$key]['quantity'] = 1;
					$parameters['service_id'][]= $parameters['visit_type'][$key]['service_doctor_id'];
				}
				$parametersNew= $parameters;
				$parametersNew['doctor_id'] = $parameters['doctor_id']['id']  ;
				$tax= KCTaxModel::get_tax($parametersNew);
				
				$parameters['tax'] = $tax['data']?? [];

				$razorpay_response = apply_filters('kivicare_create_razorpay_order', $parameters);
				if (is_array($razorpay_response) && array_key_exists('checkout_detail', $razorpay_response) && !empty($razorpay_response['status'])) {
					$razorpay_response['appointment_id'] = $appointment_id;
					$razorpay_response['data'] = $parameters;
					wp_send_json($razorpay_response);
				} else {
					(new KCAppointment())->delete(['id' => $appointment_id]);
					(new KCAppointmentServiceMapping())->delete(['appointment_id' => $appointment_id]);
					wp_send_json([
						'status' => false,
						'message' => esc_html__('Failed to create razorpay payment link', 'kc-lang'),
						'error_message' => is_array($razorpay_response) && !empty($razorpay_response['message']) ? $razorpay_response['message'] : ''
					]);
				}
				break;
			case 'paymentStripepay':
				$wpdb->update($wpdb->prefix . "kc_appointments", ['status' => 2], ['id' => $appointment_id]);
				$parameters['appointment_id'] = $appointment_id;

				$parameters['service_id']= [];
				foreach ($parameters['visit_type'] as $key => $value) {
					$parameters['visit_type'][$key]['service_id'] = $parameters['visit_type'][$key]['id'];
					$parameters['visit_type'][$key]['quantity'] = 1;
					$parameters['service_id'][]= $parameters['visit_type'][$key]['service_doctor_id'];
				}
				$parametersNew= $parameters;
				$parametersNew['doctor_id'] = $parameters['doctor_id']['id']  ;
				$tax= KCTaxModel::get_tax($parametersNew);
				
				$parameters['tax'] = $tax['data']?? [];

				$stripe_response = apply_filters('kivicare_create_stripepay_order', [
					'is_for_API' => true
				],$parameters,$appointment_id);
				if (is_array($stripe_response) && array_key_exists('checkout_detail', $stripe_response) && !empty($stripe_response['status'])) {
					$stripe_response['appointment_id'] = $appointment_id;
					$stripe_response['data'] = $parameters;
					$stripe_response['checkout_detail']['currency']= $stripe_response['checkout_detail']['currency']['id'];
					wp_send_json($stripe_response);
				} else {
					(new KCAppointment())->delete(['id' => $appointment_id]);
					(new KCAppointmentServiceMapping())->delete(['appointment_id' => $appointment_id]);
					wp_send_json([
						'status' => false,
						'message' => esc_html__('Something went Wrong please try again.', 'kc-lang'),
						'error_message' => is_array($stripe_response) && !empty($stripe_response['message']) ? $stripe_response['message'] : ''
					]);
				}
				break;

			case 'paymentWoocommerce':

				if ($parameters['id'] == null && $parameters['id'] == "") {
					$woocommerce_response = kcWoocommerceRedirect($appointment_id, $parameters);
					$response['status'] = $woocommerce_response['status'];
					$response['message'] = $message;
					$response['woocommerce_redirect'] = $woocommerce_response['woocommerce_cart_data']['woocommerce_redirect'];
				}
				break;

			default:
				# code...
				break;
		}


		if (empty($parameters['id'])) {
			if ($woocommerce_response['status']) {
				$response['status'] = $woocommerce_response['status'];
				$response['message'] = $message;

				return comman_custom_response($response);
			}
		}

		if (!empty($appointment_id) && $appointment_id !== 0) {

			$response = [
				'status'		=> true,
				'message'		=> $message,
				'notification'	=> $notification,
			];
		} else {
			$message = __('Appointment is not successfully booked.', 'kivicare-api');
			$response = [
				'status'		=> false,
				'message'		=> $message,
				'notification'	=> $notification,
			];
		}

		return comman_message_response($message);
	}

	public function deleteAppointment($request)
	{

		global $wpdb;

		$data = $this->kcaCheckAuthenticateUser($request);

		if (!$data['status']) {
			return comman_custom_response($data, 401);
		}

		$appointment_table = $wpdb->prefix . 'kc_' . 'appointments';
		$appointment_service_table = $wpdb->prefix . 'kc_' . 'appointment_service_mapping';
		$encounter_table	= $wpdb->prefix . 'kc_' . 'patient_encounters';
		$parameters = $request->get_params();

		if ($this->teleMedAddOnName()) {
			apply_filters('kct_delete_appointment_meeting', $parameters);
		}

		$results = $wpdb->delete($appointment_table, array('id' => $parameters['id']));

		if ($results) {
			$wpdb->delete($appointment_service_table, array('appointment_id' => $parameters['id']));
			$wpdb->delete($encounter_table, array('appointment_id' => $parameters['id']));
			if ($this->isKiviCareProOnName()) {
				apply_filters('kcpro_remove_appointment_event', ['appoinment_id' => $parameters['id']]);
			}
			$message = __('Appointment has been deleted successfully', 'kivicare-api');
		} else {
			$message = __('Appointment delete failed', 'kivicare-api');
		}

		return comman_message_response($message);
	}
}
