<?php

namespace Includes\Controllers\Api;

use Includes\baseClasses\KCBase;
use Includes\model\KCDoctorClinicMappingsModel;
use Includes\model\KCPatientClinicMappingsModel;
use Includes\model\KCReceptionistClinicMappingsModel;
use WP_Error;
use WP_REST_Request;
use WP_REST_Response;
use WP_REST_Server;

class KCAuthController extends KCBase
{

	public $module = 'auth';
	public $requestParam;

	public $nameSpace;

	function __construct()
	{

		$this->nameSpace = KIVICARE_API_NAMESPACE;

		add_action('rest_api_init', function () {

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/registration', array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => [$this, 'createUser'],
				'permission_callback' => '__return_true',
			));
			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/delete', array(
				'methods'             => WP_REST_Server::DELETABLE,
				'callback'            => [$this, 'deleteUser'],
				'permission_callback' => '__return_true',
			));
			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/manage-user-player-ids', array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => [$this, 'kc_manage_user_player_ids'],
				'permission_callback' => [$this, "isValidToken"],
				'args' => array(
					"player_id" => array(
						'required'    => true,
						'type'        => 'string',
						'sanitize_callback' => 'sanitize_text_field',
					),
					"logged_out" => array(
						'type'        => 'boolean',
						'default' => false,
					),
				)
			));

			register_rest_route($this->nameSpace . '/api/v1/' . $this->module, '/login', array(
				'methods'             => WP_REST_Server::CREATABLE,
				'callback'            => [$this, 'loginUser'],
				'args' => array(
					"username" => array(
						'required'    => true,
						'type'        => 'string',
						'validate_callback' => function ($param, $request, $key) {
							return is_email($param);
						},
						'sanitize_callback' => 'sanitize_email',
					),
					"password" => array(
						'type'        => 'string',
						'required'    => true,
						'sanitize_callback' => 'sanitize_text_field',
					),
				)
			));
		});
	}

	public function createUser($request)
	{

		$reqArr = $request->get_params();

		$validation = kcaValidateRequest([
			'first_name' 	=> 'required',
			'last_name' 	=> 'required',
			'user_email' 	=> 'email|required',
			// 'user_pass' 	=> 'required',
			'mobile_number' => 'required',
			'dob'           => 'required',
			'gender'        => 'required',
			"clinic_id" => 'required',
			"role" => 'required'
		], $reqArr);


		if (count($validation)) {
			return comman_message_response($validation[0], 400);
		}
		$reqArr['user_login'] = kcaGenerateUsername($reqArr['first_name']);
		if (!isset($reqArr['user_pass'])) {
			$reqArr['user_pass'] = kcaGenerateString(12);
		}
		$res = wp_insert_user($reqArr);

		if (isset($res->errors)) {
			return comman_message_response(kcaGetErrorMessage($res), 400);
		}

		wp_update_user([
			'ID' => $res,
			'first_name' => $reqArr['first_name'],
			'last_name' => $reqArr['last_name']
		]);

		$users = get_userdata($res);

		$users->set_role([
			'patient' => $this->getPatientRole(),
			'doctor' => $this->getDoctorRole(),
			'receptionist' => $this->getReceptionistRole()
		][$reqArr['role']]);

		if ($users) {

			$user_email_param = array(
				'username'            => $users->user_login,
				'user_email'          => $users->user_email,
				'password'            => $reqArr['user_pass'],
				'email_template_type' => 'patient_register'
			);
			kcaSendEmail($user_email_param);
		}
		$temp = [
			"mobile_number" => $reqArr["mobile_number"],
			"gender"        => $reqArr["gender"],
			"dob"           => $reqArr["dob"],
			"address"       => $reqArr["address"],
			"city"          => $reqArr["city"],
			// "state"         => $reqArr["state"],
			"country"       => $reqArr["country"],
			"postal_code"   => $reqArr["postal_code"],
			"blood_group"   => $reqArr["blood_group"],
		];
		$data = $this->kcaCheckAuthenticateUser($request);

		update_user_meta($users->ID, "basic_data", json_encode($temp));
		if (kcPatientUniqueIdEnable('status')) {
			update_user_meta($users->ID, 'patient_unique_id', generatePatientUniqueIdRegister());
		}

		$basic_data  = get_user_meta($users->ID, "basic_data", true);

		$reqArr["clinic_id"] = !isset($reqArr["clinic_id"])  ?		kcGetDefaultClinicId() : $reqArr["clinic_id"];
		[
			'patient' => function () use ($reqArr, $users, $data) {
				$added_by = null;
				if ($data['status']) {
					$added_by = $data['user_id'];
				}
				update_user_meta($users->ID, "patient_added_by", $added_by);

				$patient_clinic = KCPatientClinicMappingsModel::find_where(['patient_id' => $users->ID]);
				if (is_null($patient_clinic) || isset($reqArr["clinic_id"])) {
					KCPatientClinicMappingsModel::insert(['patient_id' => $users->ID, "clinic_id" => $reqArr["clinic_id"]]);
				} else {
					$patient_clinic->update(['clinic_id' => $reqArr["clinic_id"]]);
				}
			},
			'doctor' => function () use ($reqArr, $users) {
				$doctor_clinic = KCDoctorClinicMappingsModel::find_where(['doctor_id' => $users->ID]);
				if (is_null($doctor_clinic) || isset($reqArr["clinic_id"])) {
					KCDoctorClinicMappingsModel::insert(['doctor_id' => $users->ID, "clinic_id" => $reqArr["clinic_id"]]);
				} else {
					$doctor_clinic->update(['clinic_id' => $reqArr["clinic_id"]]);
				}
			},
			'receptionist' => function () use ($reqArr, $users) {
				$receptionist_clinic = KCReceptionistClinicMappingsModel::find_where(['receptionist_id' => $users->ID]);
				if (is_null($receptionist_clinic) || isset($reqArr["clinic_id"])) {
					KCReceptionistClinicMappingsModel::insert(['receptionist_id' => $users->ID, "clinic_id" => $reqArr["clinic_id"]]);
				} else {
					$receptionist_clinic->update(['clinic_id' => $reqArr["clinic_id"]]);
				}
			}

		][$reqArr['role']]();


		$user_data = [
			"first_name" 	=> $users->first_name,
			"last_name" 	=> $users->last_name,
			"user_email" 	=> $users->user_email,
			"user_login" 	=> $users->user_login,
		];
		$response["data"] = (object) array_merge($user_data, (array) json_decode($basic_data));
		$response["message"] = __("Register succesfully");
		return comman_custom_response($response);
	}

	public function loginUser(WP_REST_Request $request)
	{
		error_log(json_encode($request->get_params()));
		if ($wp_user = get_user_by('email', $request->get_param('username'))) {
			$kc_user_roles =	array_intersect($this->KCGetRoles(), $wp_user->roles);
			if (empty($kc_user_roles)) {
				return comman_message_response(__('The provided credential has no Kivicare user role.'), 200);
			}
			$creds = array(
				'user_login'    => $wp_user->user_email,
				'user_password' => $request->get_param('password'),
				'remember'      => false
			);
			$user = wp_signon($creds, false);
			if (is_wp_error($user))
				return comman_message_response($user->get_error_message(), 200);
			else {
				
				
				// FireBase Notification Token
				$firebase_tokens = [];
				if ($user_firebase_tokens = get_user_meta($user->ID, KIVI_CARE_PREFIX . 'firebase_tokens', true)) {
					$firebase_tokens = $user_firebase_tokens;
				}
				if ($request->has_param('firebase_token') && !empty($request->get_param('firebase_token'))) {
					array_push($firebase_tokens, $request->get_param('firebase_token'));
				}
				update_user_meta($user->ID, KIVI_CARE_PREFIX . 'firebase_tokens', array_unique($firebase_tokens));


				

				wp_clear_auth_cookie();
				header_remove('Set-Cookie');
				wp_set_current_user($user->ID);
				wp_set_auth_cookie($user->ID);

				$data = [
					"nounce" => wp_create_nonce('wp_rest'),
					"display_name" => $user->display_name,
					"user_name" => $user->user_login,
					"email" => $user->user_email,
				];

				$user_info = get_userdata($user->ID);
				$basic_data  = get_user_meta($user->ID, "basic_data", true);

				$module_config = kcaGetModules();
				$date_format =  kivicareConvertMomentFormatToPhp(get_option(KIVI_CARE_PREFIX . 'date_format', "D-MM-YYYY"));

				// if( isset($user->user_status) && (int)$user->user_status === 4 ) {
				// 	$message = 'Login has been disabled. please contact you system administrator.';

				// 	return comman_message_response($message,401);
				// }
				$profile = (array) json_decode($basic_data);

				$profile['dob'] = kc_wp_date($date_format, strtotime($profile['dob']));

				if (isset($profile['specialties']) && !empty($profile['specialties'])) {
					$profile['specialties'] = array_map(function ($row) use ($date_format) {
						if (!is_null($row->created_at))
							$row->created_at = kc_wp_date($date_format, strtotime($row->created_at));
						return $row;
					}, $profile['specialties']);
				}

				/*
		if ( array_key_exists( 'profile_image', $profile ) && !empty($profile['profile_image']) ) {
			$profile['profile_image'] = wp_get_attachment_url( $profile['profile_image'] );
		}else {
			$profile['profile_image'] = null;
		}
		*/
				$data['first_name'] = $user_info->first_name;
				$data['last_name']  = $user_info->last_name;
				$data['user_id']    = $user->ID;
				$data['role']		= getUserRole($user->ID);
				$data = array_merge($data, kc_woo_generate_client_auth('kivicare_app', $user->ID, 'read_write'));

				$profile['profile_image'] = kcaGetProfileImage($data['role'], $user->ID);
				if ($data['role'] == 'doctor' || $data['role'] == 'receptionist') {
					$clinic_data['clinic'] = getUserClinicMapping($data);
					$data = array_merge($data, $clinic_data);
				} else {
					$patient_clinic_mappings_model = KCPatientClinicMappingsModel::find_where(['patient_id' => $user->ID]);
					$clinic_id = !is_null($patient_clinic_mappings_model) ?  $patient_clinic_mappings_model->clinic_id : kcGetDefaultClinicId();
					$clinic_data['clinic'] = [kcaGetClinic($clinic_id)];
					$data = array_merge($data, $clinic_data);
				}
				$data['isTeleMedActive'] = $this->teleMedAddOnName();

				if ($data['role'] == 'doctor') {

					if ($this->teleMedAddOnName()) {
						$config_data = kcaGetZoomConfig($user->ID);
						$data = array_merge($data, $config_data);
					}
				}

				$data['isKiviCareProOnName'] = $this->isKiviCareProOnName();

				if ($data['isKiviCareProOnName']) {
					$enableEncounter = json_decode(get_option(KIVI_CARE_PREFIX . 'enocunter_modules'));
					$enablePrescription = json_decode(get_option(KIVI_CARE_PREFIX . 'prescription_module'));

					$data['enocunter_modules'] = isset($enableEncounter->encounter_module_config) ? $enableEncounter->encounter_module_config : [];
					$data['prescription_module'] = isset($enablePrescription->prescription_module_config) ? $enablePrescription->prescription_module_config : [];

					$get_googlecal_config = get_option(KIVI_CARE_PREFIX . 'google_cal_setting', true);
					$data['is_enable_google_cal'] = $get_googlecal_config['enableCal'] == 1 ? 'on' : 'off';

					$get_patient_cal_config = get_option(KIVI_CARE_PREFIX . 'patient_cal_setting', true);
					$data['is_patient_enable'] = $get_patient_cal_config == 1 ? 'on' : 'off';
					$data['google_client_id'] = trim($get_googlecal_config['client_id']);

					if ($data['role'] == 'doctor' || $data['role'] == 'receptionist') {
						$doctor_enable = get_user_meta($data['user_id'], KIVI_CARE_PREFIX . 'google_cal_connect', true);

						$data['is_enable_doctor_gcal'] = ($doctor_enable == 'off' || empty($doctor_enable)) ? 'off' : 'on';
					}
				}
				$data = array_merge($data, (array) $module_config);
				$data = (object) array_merge($data, $profile); //(array) json_decode( $basic_data ) );

				return comman_custom_response(['data' => $data, "message" => __("The user logged in successfully.", 'kivicare-api')]);
			}
		}

		return comman_message_response(__('No user available.', 'kivicare-api'), 404);
	}

	public function deleteUser(WP_REST_Request $request)
	{
		if (wp_delete_user(get_current_user_id())) {
			return comman_custom_response(array("message" => __("Your Account Delete Successfully", 'kivicare-api')));
		}
		return comman_custom_response(array("message" => __("Some Thing Went Wrong", 'kivicare-api')));
	}
	/**
	 * The function adds a new player ID to the user's existing list of player IDs and returns a success
	 * or error message.
	 * 
	 * @param WP_REST_Request request The  parameter is an instance of the WP_REST_Request class,
	 * which is used to handle REST API requests in WordPress. It contains information about the current
	 * request, such as the HTTP method, headers, and query parameters.
	 * 
	 * @return This function returns a message response indicating whether the player ID update was
	 * successful or not. If the update was successful, it returns a success message with a status code of
	 * 200. If the update was not successful, it returns an error message with a status code of 200.
	 */
	public function kc_manage_user_player_ids(WP_REST_Request $request)
	{
		

		if ($player_ids = get_user_meta($this->user['id'], KIVI_CARE_PREFIX . 'firebase_tokens', true)) {
			if ($this->requestParam['logged_out']) {
				unset($player_ids[array_search($this->requestParam['player_id'], $player_ids)]);
			} elseif (isset($this->requestParam['player_id']) && !empty($this->requestParam['player_id'])) {
				array_push($player_ids, $this->requestParam['player_id']);
			}
			if (update_user_meta($this->user['id'], KIVI_CARE_PREFIX . 'firebase_tokens', array_unique($player_ids)) > 0) {
				return comman_message_response(sprintf(__("Player ID %s Successfully", 'kivicare-api'), $this->requestParam['logged_out'] ? __("Removed", "kivicare-api") : __("Update", "kivicare-api")), 200);
			}
		}

		return comman_message_response(__("SomeThing Went Wrong", 'kivicare-api'), 200);
	}
}
